;---------------------------------------------------------------
; RetroForth/Native 8.1
;
; This replaces the MBR and partition table
;---------------------------------------------------------------

OS_MENU_SECTOR:
ORG 0xFC00
format binary
use16
prepare_for_real_mode:

set_up_segment_registers:
	xor	ax, ax
	mov	ds, ax
	mov	es, ax
	mov	fs, ax
	mov	gs, ax
set_up_stack:		; at 0xFE00..0xFFFF
	cli
	mov	ss, ax
	mov	sp, ax
	sti

move_boot_sector:	; to 0xFC00..0xFDFF
	mov	si, 0x7C00
	mov	di, 0xFC00
	mov	cx, 256
	rep	movsw

clear_screen:
	mov	ax, 0x0003
	int	0x10

boot_retro:
	mov	ax, 0x0200+62 ; Service 2 (read) -- 62 sectors
	mov	bx, 0x7E00    ; Buffer address
	mov	cx, 0x0002    ; Cylinder 0, sector 2
	mov	dx, 0x0080    ; Head 0, drive 0x80
	int	0x13

	; Switch to protected mode
	; Clear interrupts
	cli

	; Enable A20
	call	a20enable

	; Load the GDT, and set the pmode bit
	lgdt	[gdt]
	mov	eax, cr0
	inc	eax
	mov	cr0, eax

	; Jump to pmode code
	jmp	dword 8:pmode	

; Enable A20 gate
; Taken from F by Robert Oestling
a20enable:
	mov	al,0xd1
	out	0x64,al
	call	a20wait
	mov	al,0xdf
	out	0x60,al
	call	a20wait
	mov	al,0xff
	out	0x64,al
a20wait:
	in	al,0x64
	jmp	.l0
.l0:	and	al,2
	jnz	a20wait
	ret

; The previous GDT worked, but didn't conform to
; how it was supposed to work. So, I took the GDT
; from F and fixed it up to work here. It is still
; a mystery to me why the old GDT worked.
gdt:
	dw	(.end-.dummy)
	dd	.dummy
	dw	0
.dummy:	dw	0,0,0,0
	dw	0xffff,0x0000
	db	0x00,0x9a,0xcf,0x00
	dw	0xffff,0x0000
	db	0x00,0x92,0xcf,0x00
.end:

; PROTECTED MODE CODE STARTS HERE!
use32

; Set up and get to kernel
pmode:
	; Init segments (again) and stack
	mov	ax, 0x10
	mov	ds, ax
	mov	es, ax
	mov	ss, ax
	mov	esp, 0x50000
	
	; Move kernel from 0x7e00 to 0x110000
	; (stupid real mode stuff, argh, I hate it)
	cld
	mov	esi, 0x7e00
	mov	edi, 0x11000
	mov	edx, edi
	mov	ecx, (_end-kernel)/4
	rep	movsd

	; Call the kernel, and hang if theres a problem
	call	edx
	jmp	$


print16:
        ; Grab address of string
        pop     si

        ; Grab byte, check for zero
.again: lodsb
        or      al, al
        jz      .done

        ; Print the char, and loop
        mov     ah, 0x0e
        mov     bx, 14
        int     0x10
        jmp     .again

        ; String offset is now return address
.done:  jmp     si

times   510-($-0xFc00)	    db 0
dw 0xAA55


; Include precompiled image AFTER the 512 bytes
kernel:
	align 4
	file "native.bin"
	align 4
_end:
